/*
 * Copyright 1999 ValiCert Inc. All Rights Reserved.
 *
 * $Id: vtk_ctxt.h,v 1.12 1999/11/06 10:43:42 jackk Exp $
 *
 * ValiCert Validator Toolkit
 *
 * Toolkit context routines.
 */

#ifndef VTK_CTXT_H_
#define VTK_CTXT_H_

#include <vtk_defs.h>

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Vtk_Init
 *
 * This is the toolkit initialization function. The user must use
 * this call first before using any other toolkit calls.
 * This call loads the communication library and performs
 * global initialization.
 */
VTK_FUNC Vtk_uint32 VTK_CALL Vtk_Init(void);


/*
 * Vtk_Finish
 *
 * This call releases the toolkit resources, and it should
 * be the last call made to the toolkit.
 */
VTK_FUNC Vtk_uint32 VTK_CALL Vtk_Finish(void);



/*
 * Toolkit context functions.
 * All toolkit calls require a toolkit context parameter.
 */
/*
 * Vtk_CtxtNew
 *
 * This call creates a new toolkit context. By default, the
 * toolkit context will come pre-loaded with the certificates
 * of the ValiCert Global Validation Service and its URL.
 * The ValiCert Global Validation Service will be the
 * default VA, and will be contacted using the CRT protocol.
 */
VTK_FUNC Vtk_Ctxt* VTK_CALL Vtk_CtxtNew(void);


/*
 * Vtk_CtxtDelete
 *
 * This call release memory and resources associated with a toolkit
 * context.
 */
VTK_FUNC void VTK_CALL Vtk_CtxtDelete(Vtk_Ctxt *toFree);


/*
 * Vtk_CtxtAddCerts
 *
 * This call adds certificates to the list of trusted certificates.
 * These can be VA or CA certificates, or both.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CtxtAddCerts(Vtk_Ctxt* ctxt, int type, const Vtk_CertStore *certs);


/*
 * Vtk_CtxtAddCert
 *
 * This call adds a single certificate to the list of trusted certificates.
 * This can be a VA or CA certificate, or both.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CtxtAddCert(Vtk_Ctxt *ctxt, int type, const Vtk_Cert *cert);


/*
 * Vtk_CtxtSetVaInfo
 *
 * This call sets the validation information specific to an individual CA.
 * The URL and protocol specified will be used for validations
 * of certificates issued by this CA.
 * The vaCerts is an optional parameter. If this parameter is not set, the
 * VA certificates specified through the Vtk_CtxtAddCert(s) call
 * will be used.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CtxtSetVaInfo(Vtk_Ctxt* ctxt, const Vtk_Cert *caCert,
	const char *vaUrl, enum Vtk_ValidationMech mech,
	const Vtk_ProtocolDetails *protDetails,
	const Vtk_CertStore *vaCerts);


/*
 * Vtk_CtxtSetDefaultVa
 *
 * This call sets the default VA for this toolkit context.
 * A newly created toolkit context has the Global Validation
 * Authority Service (GVAS) set as the default, CRT being the
 * default protocol. The VA and protocol specified in this
 * call will be used as the default VA for all validations that do
 * not have an explicit VA specified. (See the Vtk_CtxtSetVaInfo call.)
 * Use VTK_GVAS_URL to specify the GVAS URL.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CtxtSetDefaultVa(Vtk_Ctxt* ctxt, const char *vaUrl, enum Vtk_ValidationMech mech);


/*
 * Functions for supplying and querying of context settings
 *
 * The following context options can be controlled through the following:
 *	-HTTP proxy host
 *	-Relocation protocol
 *	-Service locator extension
 *	-Client information extension
 *	-OCSP nonce extension
 *	-Max time skew
 *	-Usage of AIA extension
 *	-Force of LDAP library load
 *	-Crl cache directory
 *	-Max crl cache time
 *	-Max crl time cache for CRLs with no NextUpdate field
 *	-LDAP search time out
 *	-Deletegated issuer callback
 *	-OCSP request signature callback
 *
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CtxtSetOption(Vtk_Ctxt* ctxt, const Vtk_CtxtOption *data);

VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CtxtGetOption(const Vtk_Ctxt* ctxt, Vtk_CtxtOption *outData);


/*
 * This call releases the memory occupied by members of the Vtk_CtxtOption
 * structure. This call should be used to release memory allocated
 * by a call to Vtk_CtxtGetOption.
 */
VTK_FUNC Vtk_uint32 VTK_CALL Vtk_CtxtOptionDeleteContent(Vtk_CtxtOption *data);


/*
 * Certificate Type definitions.
 *
 * These are used when adding X.509 certificates to the context.
 * These bit fields can be combined.
 */
#define VTK_VA_CERT		0x1 /* Trusted Validation Authority certificates */
#define VTK_TRUSTED_CA_CERT	0x2 /* Trusted Certificate Authority certificates */
#define VTK_INTERMEDIATE_CERT	0x4 /* Intermediate Certificate Authority certificate */


#ifdef __cplusplus
}
#endif

#endif
