/*
 * Copyright 1999 ValiCert Inc. All Rights Reserved.
 *
 * $Id: vtk_defs.h,v 1.21 2000/02/28 19:09:03 stefank Exp $
 *
 * ValiCert Validator Toolkit
 *
 * General definitions of types and structures
 */

#ifndef VTK_DEFS_H_
#define VTK_DEFS_H_

#include <time.h>

#ifdef _WIN32
#ifdef VTK_EXPORT 	
#define VTK_FUNC __declspec(dllexport)
#else
#define VTK_FUNC __declspec(dllimport)
#endif /* VTK_EXPORT */
#define VTK_CALLBACK __stdcall
#define VTK_CALL __cdecl
#else /* NONE WIN32 */
#define VTK_FUNC
#define VTK_CALLBACK
#define VTK_CALL
#endif


#ifdef __cplusplus
extern "C" {
#endif


/*
 * These are the definition for validation mechanisms.
 */	
enum Vtk_ValidationMech {
	VTK_VM_CRT = 1,
	VTK_VM_OCSP,
	VTK_VM_CRL
};
	

/*
 * These are the definition for data format.
 */
enum Vtk_DataFormat {
	VTK_DF_STRING = 0,
	VTK_DF_DER,
	VTK_DF_BASE64,
	VTK_DF_HEX
};
	

/*
 * These are the revocation reason codes.
 */
enum Vtk_RevocationReason {
	VTK_REV_STATUS_UNKNOWN = -1,
	VTK_REV_UNSPECIFIED = 0,
	VTK_REV_KEY_COMPROMISE,
	VTK_REV_CA_COMPROMISE,
	VTK_REV_AFFILIATION_CHANGED,
	VTK_REV_SUPERSEDED,
	VTK_REV_CESSATION_OF_OPERATION,
	VTK_REV_CERTIFICATE_HOLD,
	VTK_REV_REMOVE_FROM_CRL
};


/*
 * These are the option types available for configuration
 * in a Vtk_Ctxt structure. They are used in Vtk_CtxtSetOption
 * and Vtk_CtxtGetOption functions.
 */
enum Vtk_CtxtOptionType {
	CO_RelocationProtocol = 0,
	CO_ServiceLocatorExt,
	CO_ClientInfoExt,
	CO_OCSPNonceExt,
	CO_HTTPProxy,
	CO_MaxTimeSkew,
	CO_UseAIAData,
	CO_LoadLDAPLib,
	CO_CrlCacheDir,
	CO_MaxCrlCacheTime,
	CO_CrlNoNextUpdateCacheTime,
	CO_LDAPSearchTimeOut,
	CO_DelegatedCertCB,
	CO_OCSPSignInfo
};


/*
 * This is the URL constant for the URL of the ValiCert Global Validation
 * Service (GVAS).  It can be used to specify the GVS URL in toolkit
 * calls.
 */
VTK_FUNC const char* VTK_GVAS_URL;


/*
 * These are opaque and forward structure definitions.
 */
typedef void* Vtk_Ctxt;
typedef unsigned char Vtk_Byte;
typedef void* Vtk_Extensions;
typedef void* Vtk_CertStore;
typedef void* Vtk_ValHdl;
typedef void* Vtk_Validation;
typedef void* Vtk_Cert;
typedef struct Vtk_CRTRespDetails_st Vtk_CRTRespDetails;
typedef struct Vtk_CRLRespDetails_st Vtk_CRLRespDetails;
typedef struct Vtk_CtxtOption_st Vtk_CtxtOption;
typedef struct Vtk_ProtocolDetails_st Vtk_ProtocolDetails;


/*
 * This is the return code type definition.
 * The toolkit uses a 32 bit unsigned integer.
 */
typedef unsigned int Vtk_uint32;
	

/*
 * The following is the general purpose buffer structure:
 */
typedef struct {
	enum Vtk_DataFormat type;
	Vtk_Byte* dPtr;
	Vtk_uint32 len;
} Vtk_Buffer;


/*
 * These are the Vtk_Buffer macros for initialization
 * and checking of content.
 */
#define	VTK_BUF_INIT(b)	(b)->type = VTK_DF_STRING; (b)->dPtr = NULL; (b)->len = 0;
#define	VTK_EMPTY_BUF(b) ((b) == NULL || (b)->dPtr == NULL || (b)->len < 1)


/*
 * X.509 Certificate structure.
 * This structure is obtained when querying for X.509 certificate details.
 */
typedef struct {
	Vtk_uint32 version;
	Vtk_Buffer serialNum;
	Vtk_Buffer signatureAlg;
	Vtk_Buffer issuer;
	time_t notBefore;
	time_t notAfter;
	Vtk_Buffer subject;
	Vtk_Buffer publicKeyAlg;
	Vtk_Buffer* issuerUniqueId;
	Vtk_Buffer* subjectUniqueId;
} Vtk_CertInfo;


/*
 * The following are structures describing detailed revocation information.
 * They include contents of OCSP/CRT responses and CRLs.
 */
/*
 * This structure presents the detailed revocation information for
 * the entire validation response (header information). It includes
 * information common to all responses. The union contains more detailed
 * information specific to each protocol. No additional OCSP
 * header information is available.
 */
typedef struct
{
	enum Vtk_ValidationMech type;
	Vtk_uint32 version;
	Vtk_Buffer *issuerIdByName;
	Vtk_Buffer *issuerIdByKey;
	time_t issueTime;
	Vtk_Extensions *extensions;
	union
	{
		Vtk_CRTRespDetails	*crt;
		Vtk_CRLRespDetails	*crl;
	} d;
} Vtk_ValRespDetails;


/*
 * This is additional revocation header information for a CRT response.
 */
struct Vtk_CRTRespDetails_st
{
	Vtk_uint32 minVersionToRead;
	time_t nextUpdate;
	time_t validUntil;
	Vtk_Buffer hashAlgorithm;
	Vtk_uint32 fullTreeLeafCount;
};


/*
 * This is additional revocation header information for a CRL response.
 */
struct Vtk_CRLRespDetails_st
{
	time_t nextUpdate;
};


/*
 * The following is detailed revocation information for a single certificate.
 * A revocation response or CRL will contain this information
 * for all the certificates they describe.
 */
typedef struct
{
	enum Vtk_ValidationMech type;
	Vtk_uint32 certStatus;
	time_t thisUpdate;
	time_t nextUpdate;
	time_t revocationTime;
	enum Vtk_RevocationReason revocationReason;
	Vtk_Extensions *extensions;
} Vtk_ValRespSingleCertDetails;


/*
 * The following is individual validation query information. The
 * application can obtain individaul query information in order to
 * perform its own retrieval of the information from the
 * VA.
 */
typedef struct
{
	enum Vtk_ValidationMech type;
	char *host;
	int port;
	char *url;
	Vtk_ProtocolDetails *protocolDetails; 
	Vtk_Buffer request;
	Vtk_Buffer response;
	void *vtkPrivateData;
} Vtk_ValQuery;


/*
 * The following is the structure used to represent X.509 extensions.
 * This includes extensions from certificates, CRLs, OCSP and CRT protocols.
 */
typedef struct
{
	Vtk_Buffer oid;
	int critical;
	Vtk_Buffer value;
	void *vtkPrivateData;
} Vtk_Extension;


/*
 * The following is the certificate chain building callback function typedef.
 * The caller to Vtk_ValiDationAddCertChain can provide a function pointer
 * which will be called every time a new link in the certificate chain
 * is discovered.
 * The user can have the function return the following values:
 *	VTK_OK  - Ok, add the certificate.
 *      !VTK_OK - No, do not use this certificate; search for others.
 */
typedef int (VTK_CALLBACK *Vtk_ChainBuildCallback)(void *userHdl, const Vtk_Cert *nextCert);


/*
 * This is the callback used during the check of OCSP and CRT responses in
 * delegated trust models.
 *
 * The callback is made when the signer of the response is not directly
 * part of the trusted VA or CA certificate store.
 * This allows applications to examine and reject the certificate used
 * to validate the certificate found in the response.
 * The user can have the function return the following values:
 *	VTK_OK -  Ok, certificate is good.
 *      !VTK_OK - No, do not use this certificate for validation of the response.
 *
 * This callback is set throught the Vtk_CtxtSetOption.
 */
typedef int (VTK_CALLBACK *Vtk_DelegatedCertCallback)(void *userHdl,
			const Vtk_Cert *delegatedCert, 
			const Vtk_Cert *trustedCert);


/*
 * The following is the callback for the signing of OCSP requests.
 * The OCSP requests can be optionally signed. If this callback
 * is set, it will enable the application to add signatures to
 * the outgoing requests.
 * This callback is set through the Vtk_CtxtSetOption
 * The user can have the function return the following values:
 *	VTK_OK  - Ok, signature generated successfully.
 *      !VTK_OK - Error, do not proceed.
 */
typedef int (VTK_CALLBACK *Vtk_OCSPSignCallback)(void* userHdl,
			Vtk_Buffer *digest,
			Vtk_Buffer *sigOut,
			int maxSigOutBuf,
			int padding);



/*
 *
 * The structures listed below are used in conjunction with the CRL protocol.
 *
 */
/*
 * These are definitions of different CRL data formats.
 */	
enum Vtk_DataType {
	VTK_DT_CRL = 0,
	VTK_DT_PKCS7
};

/*
 * The following is a detailed specification of the CRL retrieval mechanism.
 */
typedef struct
{
	enum Vtk_DataFormat encoding;
	enum Vtk_DataType type;
} Vtk_CRLProtocolDetails;


/*
 * The following is a detailed protocol specification - currently
 * only used for CRLs.
 */
struct Vtk_ProtocolDetails_st
{
	enum Vtk_ValidationMech type;
	union
	{
		Vtk_CRLProtocolDetails	crl;
	} d;
};


/*
 * The following are structures used with Vtk_CtxtSetOption
 * and Vtk_CtxtGetOption functions.
 */

/*
 * This structure is used for setting of the Proxy information context
 * option.  Use Vtk_CtxtOptionDeleteContent to release the structure
 * when it is returned from Vtk_CtxtGetOption.
 */
typedef struct
{
	char *host;
	int port;
} Vtk_ProxyInfo;


/*
 * This structure is used for setting of the callback function
 * for the supplied context.
 * This is unsed for the following callbacks:
 *	Vtk_DelegatedCertCallback
 */
typedef struct
{
	union {
		Vtk_DelegatedCertCallback DelCertCB;
	} f;
	void* userHdl;
} Vtk_Callback;


/*
 * This structure is used for setting of the OCSP request
 * signer information and callback function for the supplied
 * context. It includes the certificate of the signer, which
 * will be included in the request. The toolkit copies the
 * certificate, and the application should release it.
 */
typedef struct
{
	Vtk_OCSPSignCallback OCSPSignCB;
	void* userHdl;
	Vtk_Cert* signerCert;
} Vtk_OCSPSignInfo;



/*
 * These are the logging types available for configuration
 * logging information
 */
enum Vtk_CtxtLogType {
	LOG_Error = 0,
	LOG_Info,
	LOG_Debug,
	LOG_VaData
};

/*
 * The following functions are the callbacks for the message logging .
 * If the callbacks are set, they will replace default functions
 * for openning, closing and writting log information.
 * The callbacks are part of Vtk_LogOptions and are set through the Vtk_OpenLog
 * The user can have the Vtk_OpenLogCallback function return the following values:
 *	VTK_OK  - Ok, 
 * !VTK_OK  - Error, do not proceed.
 */
typedef Vtk_uint32 (VTK_CALLBACK *Vtk_OpenLogCallback)(const Vtk_Ctxt *pCtxt, void** userHdl);
typedef void  (VTK_CALLBACK *Vtk_CloseLogCallback)(const Vtk_Ctxt *pCtxt, void* userHdl);
typedef void  (VTK_CALLBACK *Vtk_WriteLogCallback)(const Vtk_Ctxt *pCtxt, 
												   enum Vtk_CtxtLogType type, 
												   const char *pMsg,
												   void* userHdl);



#define VTK_LOG_MODE_DEFAULT    0x0000
#define VTK_LOG_MODE_OVERWRITE  0x0001
#define VTK_LOG_MODE_FLUSH      0x0002

/*
   Logging configuration options.
   logLevel:

   Log Level        Types of Messages
   ----------------------------------
   LOG_Error        LOG_Error
   LOG_Info         LOG_Error + LOG_Info
   LOG_Debug        LOG_Error + LOG_Info + LOG_Debug
   LOG_VaData       LOG_Error + LOG_Info + LOG_Debug + LOG_VaData


   logMode:

   Bit              Meaning
   -----------------------------------
   1.(lowest)       0-Append,  1-Overwrite
   2.               0-NoFlush, 1-Flush 
 */
typedef struct {
	enum Vtk_CtxtLogType logLevel;  /* Log Level */
	Vtk_uint32 logMode;             /* Log Mode  */
	Vtk_Buffer logFileName;         /* Path+Name of the Log File */
	Vtk_OpenLogCallback  openLogCB;    /* Pointer to function openning the log */
	Vtk_CloseLogCallback closeLogCB;   /* Pointer to function closing the log */
	Vtk_WriteLogCallback writeLogCB;   /* Pointer to main logging function */
} Vtk_LogOptions;




/*
 * This is the structure for passing Vtk_Ctxt option data
 * to the Vtk_CtxtSetOption function.
 */
struct Vtk_CtxtOption_st
{
	enum Vtk_CtxtOptionType option;
	union {
		char* aChar;
		int anInt;
		Vtk_ProxyInfo *aProxyInfo;
		Vtk_Callback *aCB;
		Vtk_OCSPSignInfo *aSignInfo;
	} d;
};


#ifdef __cplusplus
}
#endif


#endif /* VTK_DEFS_H_ */


