/*
 * Copyright 1999 ValiCert Inc. All Rights Reserved.
 *
 * $Id: vtk_error.h,v 1.18 2000/03/02 05:24:43 stefank Exp $
 *
 * ValiCert Validator Toolkit
 *
 * Error constants and status bit field definitions.
 */

#ifndef VTK_ERRS_H_
#define VTK_ERRS_H_

#include <vtk_defs.h>


#define VTK_OK		0

#define VTK_ERR_BAD_LEAF_COUNT				1
#define VTK_ERR_UNSUPPORTED_DATA_TYPE			2
#define VTK_ERR_BAD_SIGNATURE				3
#define VTK_ERR_OUT_OF_MEMORY				4
#define VTK_ERR_BAD_FILE				5
#define VTK_ERR_INDEX_OUT_OF_BOUNDS			6
#define VTK_ERR_INTERNAL				7
#define VTK_ERR_BAD_ASN1				8
#define VTK_ERR_BAD_LEAF_POSITION			9
#define VTK_ERR_UNABLE_TO_GET				10
#define VTK_ERR_INCOMPATIBLE_VERSION			11
#define VTK_ERR_BAD_NONCE				12
#define VTK_ERR_BAD_CRL					13
#define VTK_ERR_NO_TRUSTED_CA				14
#define VTK_ERR_WRONG_CRL				15
#define VTK_ERR_BAD_PARMS				16
#define VTK_ERR_OP_NOT_SUPPORTED			17
#define VTK_ERR_NO_CERTS_TO_VALIDATE			18
#define VTK_ERR_BAD_URL					19
#define VTK_ERR_NO_DATA					20
#define VTK_ERR_NO_VALHDL_INFO				21
#define VTK_ERR_BAD_VALHDL				22
#define VTK_ERR_UNKNOWN_VA				23
#define VTK_ERR_LDAP_SIZE_LIMIT_EXCEEDED		24
#define VTK_ERR_LDAP_SEARCH				25
#define VTK_ERR_CRL_FORMAT				26
#define VTK_ERR_CONNECT					27
#define VTK_ERR_ERR_SEND_REQ				28
#define VTK_ERR_SOCK_READ				29
#define VTK_ERR_NULL_RESPONSE				30
#define VTK_ERR_BAD_RESPONSE				31
#define VTK_ERR_BAD_MIME_HDR				32
#define VTK_ERR_VA_REFUSED				33
#define VTK_ERR_INIT_COMM				34
#define VTK_ERR_CERTS_DO_NOT_MATCH			35
#define VTK_ERR_VA_NO_CER_INFO				36
#define VTK_ERR_CERT_PATH_TOO_DEEP			37
#define VTK_ERR_NO_ISSUER_CERT				38
#define VTK_ERR_URL_TO_LONG				39
#define VTK_ERR_DECODING_PKCS7				40
#define VTK_ERR_CTXT_OPTION_NOT_SET			41
#define VTK_ERR_UNABLE_TO_LOAD_LDAP_LIB			42
#define VTK_ERR_UNABLE_TO_LOAD_LDAP_FUNCS		43
#define VTK_ERR_LDAP_AUTH				44
#define VTK_ERR_LDAP_NOT_FOUND				45
#define VTK_ERR_USER_CALLBACK				46
#define VTK_ERR_NOT_FUND				47
#define	VTK_ERR_SRV_MALFORMED_REQUEST			48
#define VTK_ERR_SRV_INTERNAL_ERROR			49
#define VTK_ERR_SRV_TRY_LATER				50
#define VTK_ERR_SRV_SIG_REQUIRED			51
#define VTK_ERR_SRV_UNAUTHORIZED			52
#define VTK_ERR_OPEN_LOG_FILE				53
#define VTK_ERR_LOG_ALREADY_OPEN			54
#define VTK_ERR_CRYPTO_LIB_INIT				55


/*
 * Certificate status values
 *
 * The status values are defined as bit fields. A single
 * status value can represent multiple status codes.
 */
#define VTK_STATUS_OK				0x1
#define VTK_STATUS_UNKNOWN			0x2
#define VTK_STATUS_REVOKED			0x4
#define VTK_STATUS_CRL_EXPIRED			0x8
#define VTK_STATUS_CRL_NOT_YET_VALID		0x10
#define VTK_STATUS_RESP_EXPIRED			0x20
#define VTK_STATUS_RESP_NOT_YET_VALID		0x40


#ifdef __cplusplus
extern "C" {
#endif

/*
 * Vtk_ErrorToString 
 *
 * This is the return code translation routine. It returns a static
 * string representation of the specified return code.
 * The return value must not be deleted.
 * Note this function uses an internal static buffer which makes it
 * not MT-safe. See Vtk_ErrorToString_r
 */
VTK_FUNC const char* VTK_CALL Vtk_ErrorToString(Vtk_uint32 errorCode);


/*
 * Vtk_ErrorToString_r
 * 
 * Converts the Toolkit return code to a string. (this is MT-Safe version)
 * The caller must supply a buffer buf with length buflen to store the result
 */
VTK_FUNC const char* Vtk_ErrorToString_r(Vtk_uint32 errorCode, char *buf, int buflen);


/*
 * Vtk_StatusToStrings
 *
 * This is the status bit field translation routine. It returns
 * a NULL terminated array of strings containing textual
 * representation of all the errors contained in a status
 * value. The end of the array is indicated by a NULL entry.
 * The memory occupied by the array needs to be released
 * using the Vtk_StatusStringsDelete routine.
 */
VTK_FUNC char ** VTK_CALL Vtk_StatusToStrings(Vtk_uint32 status);


/*
 * Vtk_statusStringsDelete
 *
 * This call releases memory occupied by a status strings array
 * returned from Vtk_StatusToStrings.
 */
VTK_FUNC void VTK_CALL Vtk_StatusStringsDelete(char** statusStrings);



/*
 * Vtk_OpenLog
 *
 * This function sets passed logging options into context and  
 * calls OpenLog callback function from passed log options.
 * If no OpenLog callback function was set, default Toolkit
 * implementation opens text log file.
 * Function returns value returned by Vtk_OpenLogCallback
 * An application must call this function to initialize
 * logging functionality.
 * 
 */
VTK_FUNC Vtk_uint32 VTK_CALL Vtk_OpenLog(Vtk_Ctxt *pCtxt, 
		const Vtk_LogOptions *pLogOptions);


/*
 * Vtk_CloseLog
 *
 * This function first calls CloseLog callback function 
 * which was set in log options (via Vtk_OpenLog). If no 
 * callback was set, default implementation is called 
 * which closes the log file.
 * Function also resets and frees logging options in context.
 *
 * An application should call this function to close
 * logging functionality.
 * 
 */
VTK_FUNC void VTK_CALL Vtk_CloseLog(Vtk_Ctxt *pCtxt);


/*
 * Vtk_WriteLog
 *
 * Writes log messages into log file, which was previously
 * open by Vtk_OpenLog function
 * This function is used by Toolkit to write all logging 
 * messages. An application can optionaly use it to put
 * extra information into the log file.
 * 
 * This function can be optionally replaced by user defined 
 * function defined in log options
 */
VTK_FUNC void VTK_CALL
Vtk_WriteLog(const Vtk_Ctxt *pCtxt, enum Vtk_CtxtLogType type, const char *pMsg);





#ifdef __cplusplus
}
#endif


#endif /* VTK_ERRS_H_ */


