/*
 * Copyright 1999 ValiCert Inc. All Rights Reserved.
 *
 * $Id: vtk_valid.h,v 1.15 1999/12/18 09:17:26 jackk Exp $
 *
 * ValiCert Validator Toolkit
 *
 * Validation query routines
 */

#ifndef VTK_VALIDATION_H_
#define VTK_VALIDATION_H_

#include <vtk_defs.h>

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Validation Structure and Functions
 *
 * This call performs OCSP, CRT, or CRL validation checking for the specified
 * certificates. A single Validation may result in multiple queries
 * to different VAs using different protocols -- this is controlled
 * by a Vtk_Ctxt. By using the Vtk_ValidationSetVaInfo function, the
 * caller can constrain the query to a single protocol/VA.
 */


/*
 * Vtk_ValidationNew
 *
 * Allocates and initialized a Vtk_Validation structure.
 */
VTK_FUNC Vtk_Validation* VTK_CALL Vtk_ValidationNew(const Vtk_Ctxt *ctxt);


/*
 * Vtk_ValidationDelete
 *
 * The following function releases memory and resources previously
 * allocated by a call to Vtk_ValidationNew.
 */
VTK_FUNC void VTK_CALL Vtk_ValidationDelete(Vtk_Validation *val);


/*
 * Vtk_ValidationSetVaInfo
 *
 * This call sets specific validation protocol and VA information for
 * this validation operation. The setting provided here will
 * override any Vtk_Ctxt settings. The only supported protocols
 * are OCSP and CRT.
 *
 * The vaCerts is an optional parameter. If it is not set, the
 * VA certificates specified in the context will be used. The
 * vaCerts parameter will be copied, so callers need to release
 * it by calling Vtk_CertStoreDelete.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationSetVaInfo(const Vtk_Ctxt *ctxt, Vtk_Validation *val,
	const char *vaUrl, enum Vtk_ValidationMech mech,
	const Vtk_CertStore *vaCerts);


/*
 * Vtk_ValidationAddCert
 *
 * This call adds a certificate to be validated. It requires the caller
 * to supply the end-user certificate and its issuer certificate.
 * The optional Vtk_ValHdl can be used to set protocol-specific
 * extensions for this certificate validation and to
 * obtain validation status for this individual certificate.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationAddCert(const Vtk_Ctxt *ctxt, Vtk_Validation *val,
	const Vtk_Cert *cert, const Vtk_Cert *issuerCert, Vtk_ValHdl **hdl);


/*
 * Vtk_CRLValidateCert
 *
 * This call performs a CRL certificate validation. This function does
 * not require a Vtk_Validation object. Configuration data to obtain
 * the CRL is obtained from the context. The respDetails and certDetails
 * parameters are optional.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CRLValidateCert(const Vtk_Ctxt *ctxt, const Vtk_Cert *cert,
	Vtk_uint32 *status, Vtk_ValRespDetails **respDetails,
	Vtk_ValRespSingleCertDetails **certDetails);


/*
 * Vtk_ValidationAddCertRaw
 *
 * This call adds a certificate to be validated. It requires the caller
 * to supply the end-user certificate and its issuer certificate.
 * It allows the certificates to be specified in raw DER/BASE64 format.
 * The optional Vtk_ValHdl can be used to set protocol specific
 * extensions for this certificate validation and to
 * obtain validation status for this individual certificate.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationAddCertRaw(const Vtk_Ctxt *ctxt, Vtk_Validation *val,
	const Vtk_Buffer *cert, const Vtk_Buffer *issuerCert, Vtk_ValHdl **hdl);


/*
 * Vtk_ValidationAddCertChain
 *
 * This call builds a certificate chain for the specified certificate
 * and adds all the certificates to the validation structure.
 * The CA certificates stored in the Vtk_Ctxt are used while
 * constructing the chain.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationAddCertChain(const Vtk_Ctxt *ctxt, Vtk_Validation *val,
	const Vtk_Cert *cert, Vtk_ChainBuildCallback callback, void *userHdl);


/*
 * Vtk_ValidationValidate
 *
 * This call performs the actual certificate validation. This may
 * include interaction with a VA or a multiple VAs. The return
 * value indicates the success or failure of the operation.
 * The valStatus parameter will contain the aggregate validation
 * status of all the certificates in the validation.
 * Validation status per individual certificate can be
 * obtained using Vtk_ValidationGetRevStatus and
 * Vtk_ValHdlGetRevStatus functions.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationValidate(const Vtk_Ctxt *ctxt, Vtk_Validation *val, Vtk_uint32 *valStatus);


/*
 * Vtk_ValidationGetRevStatus
 *
 * This call retrieves validation status information for a single
 * certificate. This function is similar to the Vtk_ValHdlGetRevStatus.
 * The respDetails and certDetails parameters are optional. The returned
 * values must be released using Vtk_ValRespDetailsDelete and
 * Vtk_ValRespSingleCertDetailsDelete. Upon a success return value it is
 * possible that the respDetails and certDetails can be set to NULL. This
 * occurs in cases when the VA does not have any information for the
 * specified certificate. See also the Vtk_ValHdlGetRevStatus function.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationGetRevStatus(const Vtk_Ctxt *ctxt, const Vtk_Validation *val,
	const Vtk_Cert *cert, const Vtk_Cert *issuerCert,
	Vtk_uint32 *status, Vtk_ValRespDetails **respDetails,
	Vtk_ValRespSingleCertDetails **CertDetails);


/*
 * Vtk_ValHdl Routines
 *
 * Vtk_ValHdl is an auxiliary data structure used to link individual
 * certificate validation requests to their detailed revocation
 * information. The Vtk_ValHdl structure is allocated by one of the
 * Vtk_ValidationAddCert routines or by Vtk_ValidationGetValHdl.  
 * It is released by calling Vtk_ValHdlDelete(). Using Vtk_ValHdl, 
 * one can set extensions specific to a certificate and obtain validation 
 * status and details on a per certificate granularity.
 */


/*
 * Vtk_ValHdlDelete
 *
 * This call releases memory allocated to a Vtk_ValHdl structure.
 */
VTK_FUNC void VTK_CALL Vtk_ValHdlDelete(Vtk_ValHdl *hdl);

/*
 * Vtk_ValHdlGetRevStatus
 * 
 * This call retrieves validation status information for a single
 * certificate. The respDetails and certDetails parameters are optional.
 * The returned values must be released using Vtk_ValRespDetailsDelete and
 * Vtk_ValRespSingleCertDetailsDelete. Upon a success return value it is
 * possible that the respDetails and certDetails can be set to NULL. This
 * occurs in cases when the VA does not have any information for the
 * specified certificate. See also the Vtk_ValidationGetRevStatus function.
 * This call is an optimized version of Vtk_ValidationGetRevStatus routine.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValHdlGetRevStatus(const Vtk_Ctxt *ctxt, const Vtk_ValHdl *hdl,
	Vtk_uint32 *status, Vtk_ValRespDetails **respDetails,
	Vtk_ValRespSingleCertDetails **certDetails);


/*
 * Vtk_ValRespSingleCertDetailsDelete
 *
 * This call is the function for releasing memory occupied by the
 * Vtk_ValRespSingleCertDetails structure. This structure is obtained
 * from the Vtk_ValHdlGetRevStatus or Vtk_ValidationGetRevStatus call.
 */
VTK_FUNC void VTK_CALL
Vtk_ValRespSingleCertDetailsDelete(Vtk_ValRespSingleCertDetails* details);


/*
 * Vtk_ValRespDetailsDelete
 *
 * This is the function for releasing memory occupied by the Vtk_ValRespDetails
 * structure. This structure is obtained from the Vtk_ValHdlGetRevStatus
 * or Vtk_ValidationGetRevStatus call.
 */
VTK_FUNC void VTK_CALL
Vtk_ValRespDetailsDelete(Vtk_ValRespDetails *details);


/*
 * Vtk_ValidationGetQueries
 *
 * This call obtains a NULL terminated array of Validation query messages.
 * The application can then perform its own I/O with the VA. The application
 * should set the response field for each query. The returned array
 * should be deleted using Vtk_ValQueriesDelete. The toolkit will treat
 * the response buffer as read-only.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationGetQueries(const Vtk_Ctxt *ctxt, const Vtk_Validation *val,
	int *valQueryCount, Vtk_ValQuery ***queriesOut);


/*
 * Vtk_ValidationValidateFromQueries
 *
 * This call performs the equivalent of the Vtk_ValidationValidate, but
 * uses the data in the Vtk_ValQuery->response as the source of
 * validation information.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationValidateFromQueries(const Vtk_Ctxt *ctxt, Vtk_Validation *val,
	Vtk_ValQuery **valQueries, Vtk_uint32 *valStatus);


/*
 * Vtk_ValQueriesDelete
 *
 * This call releases memory occupied by an array of Vtk_ValQueries
 * obtained from a previous call to Vtk_ValidationGetQueries.
 * The response data in each Vtk_ValQuery is not released and is
 * left to the responsibility of the application.
 */
VTK_FUNC void VTK_CALL
Vtk_ValQueriesDelete(Vtk_ValQuery **queriesToDelete);


/*
 * Vtk_ValidationGetValHdl
 *
 * This function provides Vtk_ValHdl data structure for specified
 * certificate. Issuer certificate can be null if it was already
 * added to the context.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationGetValHdl(const Vtk_Ctxt *ctxt, Vtk_Validation *val,
	const Vtk_Cert *cert, const Vtk_Cert *issuerCert, Vtk_ValHdl **hdl);


/*
 * Vtk_ValidationAddReqExt
 *
 * Adds extension to OCSP/CRT validation request
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationAddReqExt(const Vtk_Ctxt *ctxt, Vtk_Validation *val, 
	const Vtk_Extension *ext);


/*
 * Vtk_ValidationAddReqExtSingleCertHdl
 *
 * Adds extension to OCSP validation request for particular certificate
 * Certificate is specified by Vtk_ValHdl
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationAddReqExtForSingleCertHdl(const Vtk_Ctxt *ctxt,
	const Vtk_ValHdl *hdl, const Vtk_Extension *ext);


/*
 * Vtk_ValidationAddReqExtSingleCert
 *
 * Adds extension to OCSP validation request for particular certificate
 * Certificate is specified by cert and issuerCert
 * Parameter issuerCert can be NULL if the issuerCert was already added into the context
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ValidationAddReqExtForSingleCert(const Vtk_Ctxt *ctxt, Vtk_Validation *val,
	const Vtk_Cert *cert, const Vtk_Cert *issuerCert, const Vtk_Extension *ext);



#ifdef __cplusplus
}
#endif


#endif /* VTK_VALIDATION_H_ */

