// Extension.cpp: implementation of the CExtension class.
//
//////////////////////////////////////////////////////////////////////

#include "Extension.h"
#include "Context.h"
#include <cassert>
#include <vtk_cert.h>
#include <vtk_error.h>

bool CExtension::m_printValue = false;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
// Extension Functions

CExtension::CExtension():m_pExt(NULL)
{
}


CExtension::CExtension(const CContext& context, const VString& oid, 
					   const unsigned char *pValue, int valueLen, int critical)
{
	assert(context.GetVtkCtxt());

	// Allocate space for this extension
	m_pExt = Vtk_ExtensionNew(context.GetVtkCtxt());
	if (m_pExt == NULL)
		throw VString("Extension initialization failed.");

	// Set oid and value into the buffer
	Vtk_Buffer oidBuf;
	oidBuf.type = VTK_DF_BASE64;
	oidBuf.dPtr = const_cast<unsigned char*>((const unsigned char*)oid.c_str());
	oidBuf.len  = oid.size();

	Vtk_Buffer valueBuf;
	valueBuf.type = VTK_DF_DER;
	valueBuf.dPtr = const_cast<unsigned char*>(pValue);
	valueBuf.len  = valueLen;

	// Initializes a Vtk_Extension data structure from supplied data.
	Vtk_uint32 nRet = Vtk_ExtensionInit(context.GetVtkCtxt(), m_pExt, &oidBuf, critical,
		&valueBuf);
	if (nRet != VTK_OK)
	{
		Vtk_ExtensionDelete(m_pExt);

		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



CExtension::~CExtension()
{
	if (m_pExt)
		Vtk_ExtensionDelete(m_pExt);
}

CExtension::CExtension(const CExtension& obj)
{
	*this = obj;
}

CExtension& CExtension::operator=(const CExtension& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Temporary the assignment is not allowed !
		// Support from Toolkit Validator is needed. 
		throw VString("Assignment of CExtension is not supported.");
	}
	return *this;
}


VString CExtension::Oid() const
{
	assert(m_pExt);

	VString strOut;
	if (m_pExt->oid.dPtr)
		strOut = (const char *)m_pExt->oid.dPtr;

	return strOut;
}

int CExtension::Critical() const
{
	assert(m_pExt);

	return m_pExt->critical;
}


void CExtension::Value(vector<unsigned char>& valueOut) const
{
	assert(m_pExt);

	for (int i=0;i<m_pExt->value.len;i++)
		valueOut.push_back(m_pExt->value.dPtr[i]);

}



ostream& operator << (ostream& os, const CExtension& obj)
{
	cout << "Critical: " << obj.Critical() << endl;
	cout << "Oid: " << obj.Oid() << endl;

	vector<unsigned char> value;
	obj.Value(value);

	int valSize = value.size();
	if (CExtension::m_printValue && valSize)
	{
		cout << "Value Length: " << valSize << endl;
		cout << "Value Data: ";
		for (int i=0; i< valSize; i++)
		{
			cout << value.at(i) << " ";
		}
		cout << endl;
	}

    return os;
}



//////////////////////////////////////////////////////////////////////
// Extensions Functions

CExtensions::CExtensions():m_pExts(NULL)
{
}


CExtensions::~CExtensions()
{
	if (m_pExts)
		Vtk_ExtensionsDelete(m_pExts);
}

CExtensions::CExtensions(const CExtensions& obj)
{
	*this = obj;
}

CExtensions& CExtensions::operator=(const CExtensions& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Temporary the assignment is not allowed !
		// Support from Toolkit Validator is needed. 
		throw VString("Assignment of CExtensions is not supported.");
	}
	return *this;
}


int CExtensions::Size(const CContext& context) const
{
	assert(context.GetVtkCtxt());

	// This function will return 0 for empty (uninitialized) CExtensions object
	int nSize = 0;
	if (m_pExts)
		nSize = Vtk_ExtensionsGetCount(context.GetVtkCtxt(), m_pExts);

	return nSize;
}


void CExtensions::GetAt(const CContext& context, int nIndex, CExtension& extOut) const
{
	assert(context.GetVtkCtxt());
	assert(m_pExts);

	// Check range
	if (nIndex < 0 || nIndex >= this->Size(context))
		throw VString("Index is out of the range.");

	Vtk_Extension *pExt;
	Vtk_uint32 nRet = Vtk_ExtensionsGetith(context.GetVtkCtxt(), m_pExts, nIndex, &pExt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Set obtained extension structure to Extension object
	extOut.m_pExt = pExt;
}


void CExtensions::GetByOid(const CContext& context, const VString& oid, CExtension& extOut) const
{
	assert(context.GetVtkCtxt());
	assert(oid.size());
	assert(m_pExts);

	// Create Buffer
	// Copy certificate from string into the buffer
	Vtk_Buffer buffer;
	VTK_BUF_INIT(&buffer);
	buffer.dPtr = new unsigned char[oid.size()+1];
	if (! buffer.dPtr)
		throw VString("Not enough memory.");

	buffer.type = VTK_DF_BASE64;
	buffer.len  = oid.size();
	strcpy((char *)buffer.dPtr, oid.c_str()); 


	Vtk_Extension *pExt;
	Vtk_uint32 nRet = Vtk_ExtensionGetByOid(context.GetVtkCtxt(), m_pExts, &buffer, &pExt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Release Buffer
	delete [] buffer.dPtr;

	// Set obtained extension structure to Extension object
	extOut.m_pExt = pExt;
}


void CExtensions::Print(ostream& os, const CContext& context) const
{
	assert(context.GetVtkCtxt());

	// Get Number of extensions
	int nNumExt = this->Size(context);
	cout << "Number of extensions: " << nNumExt << endl;

	// Get Extensions one-by-one
	for (int i=0; i<nNumExt; i++)
	{
		CExtension ext;
		this->GetAt(context, i, ext);
		os << ext << endl;
	}
}

