//
// Copyright 2000 ValiCert Inc. All Rights Reserved.
//
//
// VTKC Demo Program.
//
// This file demonstrates performing common operations with the VTKC.
//
#include <iostream>
#include <iomanip>
#include <VString.h>
#include <Certificate.h>
#include <CertificateInfo.h>
#include <CertificateStore.h>
#include <Extension.h>
#include <Context.h>
#include <Validator.h>
#include <ValRespDetails.h>
#include <ValRespSingleCertDetails.h>
#include <ValQueries.h>
#include <Log.h>


// Getting integer values from standard input
int getInt()
{
	VString str;
   	
	cin >> str;

	int retVal = 0;
	if (str.length())
		retVal = atoi(str.c_str());

	return retVal;
}


// Getting string values from standard input
VString getString()
{
	VString str;
   	cin >> str;
	return str;
}



// Demo Functions
/////////////////////////////////////////////

// Getting Certificate Information
void CertificateInfo(const VString& certFile);

// Getting Certificate Extensions
void CertificateExts(const VString& certFile);

// Performing Simple Certificate Validation
void SimpleValidation(const VString& certFile, const VString& issuerCertFile);

// Performing Certificate Validation via specified VA
void VAValidation(const VString& certFile, const VString& trustedCertFile, const VString& urlVA,
				  const VString& fileCertVA, int valMech);



// Main - drives the VTKC demo
//////////////////////////////////////////////

int main(int argc, char* argv[])
{

	enum MenuOpt { EXIT=-1, 
	               CERT_INFO = 1, CERT_EXTS, SIMPLE_CERT_VAL, VA_CERT_VAL
	             };

	int opt = 0;
	while (opt != EXIT)
	{
		cout << setw(2) << EXIT << ": Exit" << endl;
		cout << setw(2) << CERT_INFO << ": Certificate Information" << endl;
		cout << setw(2) << CERT_EXTS << ": Certificate Extensions" << endl;
		cout << setw(2) << SIMPLE_CERT_VAL << ": Simple Certificate Validation" << endl;
		cout << setw(2) << VA_CERT_VAL << ": Certificate Validation with Specified VA" << endl;

		cout << " Enter option: " << flush;
		opt = getInt();

		switch(opt)
		{
			case CERT_INFO:
			{
				// Certificate Information
				cout << " Enter BASE64 encoded certificate file: " << flush;
				VString certFile = getString();

				CertificateInfo(certFile);
				break;
			}
			case CERT_EXTS:
			{
				// Certificate Extensions
				cout << " Enter BASE64 encoded certificate file: " << flush;
				VString certFile = getString();

				CertificateExts(certFile);
				break;
			}
			case SIMPLE_CERT_VAL:
			{
				// Simple Certificate Validation
				cout << " Enter user BASE64 encoded certificate file: " << flush;
				VString certFile = getString();
				cout << " Enter issuer BASE64 encoded certificate file: " << flush;
				VString issuerCertFile = getString();

				SimpleValidation(certFile, issuerCertFile);
				break;
			}
			case VA_CERT_VAL:
			{
				// Certificate Validation via specified VA
				cout << " Enter user BASE64 encoded certificate file: " << flush;
				VString certFile = getString();
				cout << " Enter issuer BASE64 encoded certificate file: " << flush;
				VString issuerCertFile = getString();

				VString urlVA, fileCertVA;
				int valMech = 0;
				cout << " Do you want to set specific VA ? (y/n): " << flush;
				VString yesNo = getString();

				if (yesNo == "y" || yesNo == "Y")
				{
					cout << " Enter URL of the specified VA (for example http://ci.valicert.com:80/): " << flush;
					urlVA = getString();

					cout << " Enter BASE64 encoded certificate file of the specified VA: " << flush;
					fileCertVA = getString();

					cout << " Enter type of validation mechanizm (1-CRT, 2-OCSP): " << flush;
					valMech = getInt();
				}

				VAValidation(certFile, issuerCertFile, urlVA, fileCertVA, valMech);
				break;
			}
		}
	}
	return 0;
}



void SimpleValidation(const VString& certFile, const VString& issuerCertFile)
{
	// Check if any certificate was provided
	if (certFile.empty() || issuerCertFile.empty())
	{
		cout << "Missing user or issuer certificate file name.";
		return;
	}

	// Validate certificate
	///////////////////////
	try
	{
		// Create Context
		CContext ctxt;

		// Start Logging
		CLog log(LOG_VaData);
		log.OpenLog(ctxt);

		// Create both certificates
		CCertificate cert(ctxt, certFile, VTK_DF_BASE64);
		CCertificate issuerCert(ctxt, issuerCertFile, VTK_DF_BASE64);

		// Create CValidator (core validating object)
		CValidator validator(ctxt);
		CValHdl valHdl;
		validator.AddCertificate(ctxt, cert, issuerCert, &valHdl);

		// Perform validation query
		Vtk_uint32 valStatus;
		VString strStatus;
		validator.Validate(ctxt, &valStatus, &strStatus);

		// Status
		cout << endl << "Certificate status: " << strStatus << endl << endl;

		// Obtain response details via CValHdl
		///////////////////////////////////////
		if (! valHdl.IsNull())
		{
			// Retrieves validation status information for a single certificate.
			CValRespDetails respDetails;
			CValRespSingleCertDetails certDetails;
			valHdl.GetRevStatus(ctxt, &valStatus, &respDetails, &certDetails);

			// Response details
			if (! respDetails.IsNull())
			{
				cout << "Response details: " << endl;
				cout << "---------------- " << endl;
				respDetails.Print(cout, ctxt);
				cout << endl;
			}

			// Certificate details
			if (! certDetails.IsNull())
			{
				cout << "Certificate details: " << endl;
				cout << "------------------- " << endl;
				certDetails.Print(cout, ctxt);
				cout << endl;
			}
		}

		// Stop Logging
		log.CloseLog(ctxt);
	}
	catch (VString& msg)
	{
		cout << msg << endl;
	}

	return;
}



void CertificateInfo(const VString& certFile)
{
	// Check if any certificate was provided
	if (certFile.empty())
	{
		cout << "Missing user certificate file name.";
		return;
	}

	// Validated certificate
	try
	{
		// Create Context
		CContext ctxt;

		// Create both certificates
		CCertificate cert(ctxt, certFile, VTK_DF_BASE64);

		// Get cerificate information
		CCertificateInfo info;
		cert.GetCertInfo(ctxt, info);

		// Print certificate information
		cout << "Certificate Information: " << endl;
		cout << "-----------------------  " << endl;
		cout << info << endl;
	}
	catch (VString& msg)
	{
		cout << msg << endl;
	}

	return;
}


void CertificateExts(const VString& certFile)
{
	// Check if any certificate was provided
	if (certFile.empty())
	{
		cout << "Missing user certificate file name.";
		return;
	}

	// Validated certificate
	try
	{
		// Create Context
		CContext ctxt;

		// Create both certificates
		CCertificate cert(ctxt, certFile, VTK_DF_BASE64);

		// Get cerificate information
		CExtensions exts;
		cert.GetExtensions(ctxt, exts);

		// Print certificate extensions
		cout << "Certificate Extensions: " << endl;
		cout << "----------------------  " << endl;
		exts.Print(cout, ctxt);
	}
	catch (VString& msg)
	{
		cout << msg << endl;
	}

	return;
}




void VAValidation(const VString& certFile, const VString& issuerCertFile, 
				  const VString& urlVA, const VString& fileCertVA, int valMech)
{
	// Check if any certificate was provided
	if (certFile.empty() || issuerCertFile.empty())
	{
		cout << "Missing user or issuer certificate file name.";
		return;
	}

	// Check if VA parameters are correct
	bool bSpecVA = false;
	if (urlVA.empty() || fileCertVA.empty() || valMech < 1 || valMech > 2)
		cout << "Parameters of specified VA are not correct - default GVAS service will be used" << endl;
	else
		bSpecVA = true;

	// Set 
	Vtk_ValidationMech mech;
	if (valMech == 1)
		mech = VTK_VM_CRT;
	else
		mech = VTK_VM_OCSP;

	// Validate certificate
	try
	{
		// Create Context
		CContext ctxt;

		// Set new VA into the context
		if (bSpecVA)
		{
			ctxt.SetDefaultVa(mech, urlVA);

			CCertificate certVA(ctxt, fileCertVA, VTK_DF_BASE64);
			ctxt.AddCertificate(certVA, VTK_VA_CERT);
		}

		// Create user certificate and store with trusted certificates
		CCertificate cert(ctxt, certFile, VTK_DF_BASE64);
		CCertificate issuerCert(ctxt, issuerCertFile, VTK_DF_BASE64);

		// Validator
		CValHdl valHdl;
		CValidator validator(ctxt);
		validator.AddCertificate(ctxt, cert, issuerCert, &valHdl);

		// Perform validation query
		Vtk_uint32 valStatus;
		VString strStatus;
		validator.Validate(ctxt, &valStatus, &strStatus);

		// Status
		cout << endl << "Certificate status: " << strStatus << endl << endl;

		// Obtain response details via CValHdl
		////////////////////////////////////////////////////////////////////
		if (! valHdl.IsNull())
		{
			// Retrieves validation status information for a single certificate.
			CValRespDetails respDetails;
			CValRespSingleCertDetails certDetails;
			valHdl.GetRevStatus(ctxt, &valStatus, &respDetails, &certDetails);

			// Response details
			if (! respDetails.IsNull())
			{
				cout << "Response details: " << endl;
				cout << "---------------- " << endl;
				respDetails.Print(cout, ctxt);
				cout << endl;
			}

			// Certificate details
			if (! certDetails.IsNull())
			{
				cout << "Certificate details: " << endl;
				cout << "------------------- " << endl;
				certDetails.Print(cout, ctxt);
				cout << endl;
			}
		}
	}
	catch (VString& msg)
	{
		cout << msg << endl;
	}

	return;
}



